use crate::ast::{AstSpan, Const, operator};
use derive_more::Display;

#[derive(Debug, Clone, PartialEq, Display)]
pub enum VarOrConst {
    /// A [Const].
    Const(Const),

    /// A variable, just an identifier preceded by `$`, like: `$CynthiaLike`
    #[display("${_0}")]
    Var(String),
}

impl VarOrConst {
    pub fn integer(x: i32) -> Self {
        Self::Const(Const::Int(x))
    }

    pub fn float(x: f32) -> Self {
        Self::Const(Const::Flt(x))
    }

    pub fn boolean(x: bool) -> Self {
        Self::Const(Const::Bool(x))
    }

    pub fn identifier(x: impl Into<String>) -> Self {
        Self::Const(Const::Ident(x.into()))
    }

    pub fn string(x: impl Into<String>) -> Self {
        Self::Const(Const::Str(x.into()))
    }
}

#[derive(Debug)]
pub enum Expression {
    Binary(AstSpan, Box<Expression>, operator::Binary, Box<Expression>),
    Unary(AstSpan, operator::Unary, Box<Expression>),
    Leaf(AstSpan, VarOrConst),
}

impl AsRef<Expression> for Expression {
    fn as_ref(&self) -> &Expression {
        self
    }
}
