use crate::parser::Location;

#[derive(Debug, Clone, Copy, PartialEq, Eq, Hash)]
pub struct Span {
    offset: usize,
    line: usize,
    size: usize,
}

impl From<(Location, Location)> for Span {
    fn from((from, to): (Location, Location)) -> Self {
        use std::cmp::Ordering;
        let (from, to) = match PartialOrd::partial_cmp(&from, &to) {
            Some(Ordering::Less | Ordering::Equal) => (from, to),
            Some(Ordering::Greater) => (to, from),
            None => unreachable!(),
        };

        debug_assert!(from.offset <= to.offset, "error in implementation...");
        Span {
            offset: from.offset,
            line: from.line,
            size: to.offset - from.offset,
        }
    }
}

impl From<(crate::parser::Span<'_>, crate::parser::Span<'_>)> for Span {
    fn from((from, to): (crate::parser::Span<'_>, crate::parser::Span<'_>)) -> Self {
        (from.into_location(), to.into_location()).into()
    }
}
