use std::{
    num::{ParseFloatError, ParseIntError},
    str::ParseBoolError,
};

#[derive(Debug, thiserror::Error)]
pub enum Error {
    #[error("already defined spell '{0}': {1}")]
    RedefinedSpell(&'static str, &'static str),

    #[error("already defined spell argument '{0}'")]
    RedefinedSpellArgument(std::borrow::Cow<'static, str>),

    #[error("unexpect: {0}")]
    Unexpected(std::borrow::Cow<'static, str>),

    #[error("redefined: {0}")]
    Redefined(std::borrow::Cow<'static, str>),

    #[error("undefined spell '{0}'")]
    Undefined(std::borrow::Cow<'static, str>),

    #[error("can't cast '{0}' into {1:?}")]
    CantCast(crate::ast::Const, crate::types::Type),

    #[error("{0}")]
    ParseFloat(#[from] ParseFloatError),

    #[error("{0}")]
    ParseInt(#[from] ParseIntError),

    #[error("{0}")]
    ParseBool(#[from] ParseBoolError),

    #[error("empty {0}")]
    Empty(std::borrow::Cow<'static, str>),

    #[error("error in spell '{0}' instantiation: {1}")]
    BuildSpell(&'static str, Box<Error>),
}
