use crate::parser::{Location, Span};
use nom::{IResult, error::*};
use std::num::{ParseFloatError, ParseIntError};

#[derive(Debug, thiserror::Error)]
pub enum ParserError {
    #[error("{0:?}: {1}")]
    ParseFloat(Location, ParseFloatError),

    #[error("{0:?}: {1}")]
    ParseInt(Location, ParseIntError),

    #[error("{0:?} use of a reserved keyword: '{1}'")]
    UseOfReservedKeyword(Location, &'static str),

    #[error("{0:?}: nom error {1:?}")]
    Nom(Location, ErrorKind),
}

pub type ParserResult<'a, T> = IResult<Span<'a>, T, ParserError>;

impl<I> ContextError<I> for ParserError {
    fn add_context(_: I, _ctx: &'static str, other: Self) -> Self {
        other
    }
}

impl ParseError<Span<'_>> for ParserError {
    fn from_error_kind(input: Span<'_>, kind: ErrorKind) -> Self {
        Self::Nom(input.location(), kind)
    }

    fn append(_: Span<'_>, _: ErrorKind, other: Self) -> Self {
        other
    }
}

impl nom::error::FromExternalError<Span<'_>, ParseFloatError> for ParserError {
    fn from_external_error(input: Span<'_>, _: ErrorKind, e: ParseFloatError) -> Self {
        Self::ParseFloat(input.location(), e)
    }
}

impl nom::error::FromExternalError<Span<'_>, ParseIntError> for ParserError {
    fn from_external_error(input: Span<'_>, _: ErrorKind, e: ParseIntError) -> Self {
        Self::ParseInt(input.location(), e)
    }
}

impl FromExternalError<Span<'_>, ParserError> for ParserError {
    fn from_external_error(_: Span<'_>, _: ErrorKind, e: ParserError) -> Self {
        e
    }
}
