mod actor;
mod printer;

use crate::{ast::Const, error::Error};
use std::{fmt, rc::Rc};

pub use self::{actor::Actor, printer::Printer};

#[derive(Clone)]
pub enum ConstOrVnType {
    Const(Const),
    Actor(Rc<dyn Actor>),
    Printer(Rc<dyn Printer>),
}

impl fmt::Debug for ConstOrVnType {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        match self {
            Self::Const(arg0) => f.debug_tuple("Const").field(arg0).finish(),
            Self::Actor(arg0) => f.debug_tuple("Actor").field(&arg0.name()).finish(),
            Self::Printer(arg0) => f.debug_tuple("Printer").field(&arg0.name()).finish(),
        }
    }
}

impl ConstOrVnType {
    pub fn is_actor(&self) -> bool {
        matches!(self, ConstOrVnType::Actor(_))
    }

    pub fn is_printer(&self) -> bool {
        matches!(self, ConstOrVnType::Printer(_))
    }

    pub fn try_into_const(self) -> Result<Const, Error> {
        use ConstOrVnType::*;
        match self {
            Const(constant) => Ok(constant),
            Actor(vn) => Err(Error::Unexpected(Into::into(format!(
                "can't cast actor '{name}' into constant",
                name = vn.name()
            )))),
            Printer(vn) => Err(Error::Unexpected(Into::into(format!(
                "can't cast printer '{name}' into constant",
                name = vn.name()
            )))),
        }
    }
}

impl From<Const> for ConstOrVnType {
    fn from(value: Const) -> Self {
        Self::Const(value)
    }
}

impl From<Rc<dyn Actor>> for ConstOrVnType {
    fn from(value: Rc<dyn Actor>) -> Self {
        Self::Actor(value)
    }
}

impl From<Rc<dyn Printer>> for ConstOrVnType {
    fn from(value: Rc<dyn Printer>) -> Self {
        Self::Printer(value)
    }
}
