/// The struct to #[command(flatten)] inside the cli's arguments.
#[derive(clap::Args, Debug, Clone, Copy, Default)]
#[command( about = None
         , long_about = None
         , groups = [clap::ArgGroup::new("verbosity").args(["verbose", "quiet"])]
)]
pub struct VerboseChoice {
    #[arg( long
         , short  = 'v'
         , action = clap::ArgAction::Count
         , global = true
    )]
    /// Verbosity level, supply multiple times to increase log level
    verbose: u8,

    #[arg( long
         , short  = 'q'
         , action = clap::ArgAction::SetTrue
         , global = true
    )]
    /// Use to silence any log, only errors will be shown
    quiet: bool,
}

impl From<VerboseChoice> for log::Level {
    fn from(value: VerboseChoice) -> Self {
        match value {
            VerboseChoice { quiet: true, .. } => log::Level::Error,
            VerboseChoice { verbose: 0, .. } => log::Level::Error,
            VerboseChoice { verbose: 1, .. } => log::Level::Warn,
            VerboseChoice { verbose: 2, .. } => log::Level::Info,
            VerboseChoice { verbose: 3, .. } => log::Level::Debug,
            VerboseChoice { .. } => log::Level::Trace,
        }
    }
}

impl From<VerboseChoice> for log::LevelFilter {
    fn from(value: VerboseChoice) -> Self {
        Into::<log::Level>::into(value).to_level_filter()
    }
}
